﻿import {
	Color,
	Vector3,
	Vector4
} from '../build/three.module.js';

const TransparentShader = {

	uniforms: {

		'Threshold': { value: 0.2 },
		'Fill': { value: false },
		'Color': { value: new Vector4(0, 0, 0, 1) },
		'Sampler': { value: null },
		'Opacity': { value: 1.0 },
		'uDirLightPos': { value: new Vector3() },
		'uDirLightColor': { value: new Color( 0xeeeeee ) },
		'uDirLightIntensity': { value: 1.0 },
		'uAmbientLightColor': { value: new Color( 0x050505 ) }

	},

	vertexShader: /* glsl */`

		varying vec3 vNormal;
		varying vec3 vRefract;
		varying highp vec2 vUV;

		void main() {

			vUV = uv;
			vec4 worldPosition = modelMatrix * vec4( position, 1.0 );
			vec4 mvPosition = modelViewMatrix * vec4( position, 1.0 );
			vec3 worldNormal = normalize ( mat3( modelMatrix[0].xyz, modelMatrix[1].xyz, modelMatrix[2].xyz ) * normal );

			vNormal = normalize( normalMatrix * normal );

			vec3 I = worldPosition.xyz - cameraPosition;
			vRefract = refract( normalize( I ), worldNormal, 1.02 );

			gl_Position = projectionMatrix * mvPosition;

		}`,

	fragmentShader: /* glsl */`

		uniform vec3 uDirLightPos;
		uniform vec3 uDirLightColor;
		uniform float uDirLightIntensity;
		uniform vec3 uAmbientLightColor;
		varying vec3 vNormal;
		varying vec3 vRefract;
		varying highp vec2 vUV;

		uniform float Threshold;
		uniform bool Fill;
		uniform vec4 Color;
		uniform sampler2D Sampler;
		uniform float Opacity;

		void main() {

			vec4 col;
	                col = texture2D( Sampler, vUV );
			col.w = clamp(col.w * Opacity, 0.0, 1.0);
			float directionalLightWeighting = max( dot( normalize( vNormal ), uDirLightPos ), 0.0) * uDirLightIntensity;
			vec3 lightWeighting = uAmbientLightColor + uDirLightColor * directionalLightWeighting;

			float intensity = smoothstep( - 0.5, 1.0, pow( length(lightWeighting), 20.0 ) );
			intensity += length(lightWeighting) * 0.2;

			float cameraWeighting = dot( normalize( vNormal ), vRefract );
			intensity += pow( 1.0 - length( cameraWeighting ), 6.0 );
			intensity = intensity * 0.2 + 0.3;
			if(Fill == true) {
				if(Threshold == 0.0) {
					col = vec4(Color.x, Color.y, Color.z, clamp(Color.w * Opacity, 0.0, 1.0));
					vec3 baseColor = vec3(col.x, col.y, col.z);
					if ( intensity < 0.50 ) {
						gl_FragColor = vec4( 2.0 * intensity * baseColor, col.w );
					}
					else {
						gl_FragColor = vec4( 1.0 - 2.0 * ( 1.0 - intensity ) * ( 1.0 - baseColor ), col.w );
					}
				}
				else {
					if(Threshold <= col.w) {
						col = vec4(Color.x, Color.y, Color.z, clamp(Color.w * Opacity, 0.0, 1.0));
					}
					else {
						col = vec4(0, 0, 0, 0);
					}
					gl_FragColor = col;
				}
			}
			else {
				vec3 baseColor = vec3(col.x, col.y, col.z);
				if(col.w < Threshold){
					gl_FragColor = vec4(0, 0, 0, 0);
				}
				else {
					if ( intensity < 0.50 ) {
						gl_FragColor = vec4( 2.0 * intensity * baseColor, col.w );
					} else {
						gl_FragColor = vec4( 1.0 - 2.0 * ( 1.0 - intensity ) * ( 1.0 - baseColor ), col.w );
					}
				}
			}

		}`

};

export { TransparentShader };
